import EventEmitter from "eventemitter3";
import * as Automerge from "@automerge/automerge";
import { ChangeOptions, Doc } from "@automerge/automerge";
import { ChannelId, PeerId } from ".";
export declare type DocumentId = string & {
    __documentId: true;
};
declare type HandleState = "loading" | "syncing" | "ready";
/**
 * DocHandle is a wrapper around a single Automerge document that allows us to listen for changes.
 */
export declare class DocHandle<T> extends EventEmitter<DocHandleEvents<T>> {
    doc: Automerge.Doc<T>;
    documentId: DocumentId;
    state: HandleState;
    constructor(documentId: DocumentId, newDoc?: boolean);
    ready(): boolean;
    loadIncremental(binary: Uint8Array): void;
    unblockSync(): void;
    updateDoc(callback: (doc: Doc<T>) => Doc<T>): void;
    __notifyChangeListeners(newDoc: Automerge.Doc<T>): void;
    __notifyPatchListeners(patch: any, //Automerge.Patch,
    before: Automerge.Doc<T>, after: Automerge.Doc<T>): void;
    value(): Promise<Automerge.Doc<T>>;
    syncValue(): Promise<Automerge.Doc<T>>;
    change(callback: (doc: T) => void, options?: ChangeOptions<T>): void;
}
export interface DocHandleMessageEvent {
    destinationId: PeerId;
    channelId: ChannelId;
    data: Uint8Array;
}
export interface DocHandleChangeEvent<T> {
    handle: DocHandle<T>;
}
export interface DocHandlePatchEvent<T> {
    handle: DocHandle<T>;
    patch: any;
    before: Automerge.Doc<T>;
    after: Automerge.Doc<T>;
}
export interface DocHandleEvents<T> {
    syncing: () => void;
    ready: () => void;
    message: (event: DocHandleMessageEvent) => void;
    change: (event: DocHandleChangeEvent<T>) => void;
    patch: (event: DocHandlePatchEvent<T>) => void;
}
export {};
//# sourceMappingURL=DocHandle.d.ts.map