import * as CBOR from "cbor-x";
import EventEmitter from "eventemitter3";
/**
 * Ephemeral Data
 * -----
 *
 * Not all that glitters is gold.
 *
 * It's useful to have a mechanism to send around short-lived data like cursor
 * positions, presence, or heartbeats. This kind of data is often high-bandwidth
 * and low-utility to persist so... this lets you communicate without that.
 *
 */
export class EphemeralData extends EventEmitter {
    // Send an ephemeral message to anyone listening to this DocHandle
    broadcast(channelId, message) {
        const cbor = CBOR.encode(message);
        this.emit("message", {
            targetId: "*",
            channelId: ("m/" + channelId),
            message: cbor,
            broadcast: true,
        });
    }
    receive(senderId, grossChannelId, message) {
        const data = CBOR.decode(message);
        const channelId = grossChannelId.slice(2);
        this.emit("data", { peerId: senderId, channelId, data });
    }
}
