import EventEmitter from "eventemitter3";
export declare type PeerId = string & {
    __peerId: false;
};
export declare type ChannelId = string & {
    __channelId: false;
};
interface AdapterOpenDetails {
    network: NetworkAdapter;
}
interface PeerCandidateDetails {
    peerId: PeerId;
    channelId: ChannelId;
}
interface PeerDetails {
    peerId: PeerId;
    channelId: ChannelId;
}
export interface OutboundMessageDetails {
    targetId: PeerId;
    channelId: ChannelId;
    message: Uint8Array;
    broadcast: boolean;
}
export interface InboundMessageDetails extends OutboundMessageDetails {
    senderId: PeerId;
}
interface DisconnectedDetails {
    peerId: PeerId;
}
export interface NetworkAdapterEvents {
    open: (event: AdapterOpenDetails) => void;
    close: () => void;
    "peer-candidate": (event: PeerCandidateDetails) => void;
    "peer-disconnected": (event: DisconnectedDetails) => void;
    message: (event: InboundMessageDetails) => void;
}
export interface NetworkEvents {
    peer: (msg: PeerDetails) => void;
    "peer-disconnected": (event: DisconnectedDetails) => void;
    message: (msg: InboundMessageDetails) => void;
}
export interface NetworkAdapter extends EventEmitter<NetworkAdapterEvents> {
    peerId?: PeerId;
    connect(url?: string): void;
    sendMessage(peerId: PeerId, channelId: ChannelId, message: Uint8Array, broadcast: boolean): void;
    join(channelId: ChannelId): void;
    leave(channelId: ChannelId): void;
}
export interface DecodedMessage {
    type: string;
    senderId: PeerId;
    targetId: PeerId;
    channelId: ChannelId;
    data: Uint8Array;
    broadcast: boolean;
}
export interface Peer extends EventEmitter<InboundMessageDetails> {
    isOpen(): boolean;
    close(): void;
    send(channelId: ChannelId, msg: Uint8Array): void;
}
export declare class NetworkSubsystem extends EventEmitter<NetworkEvents> {
    networkAdapters: NetworkAdapter[];
    myPeerId: PeerId;
    peerIdToAdapter: {
        [peerId: PeerId]: NetworkAdapter;
    };
    channels: ChannelId[];
    constructor(networkAdapters: NetworkAdapter[], peerId?: PeerId);
    addNetworkAdapter(networkAdapter: NetworkAdapter): void;
    sendMessage(peerId: PeerId, channelId: ChannelId, message: Uint8Array, broadcast: boolean): void;
    join(channelId: ChannelId): void;
    leave(channelId: ChannelId): void;
}
export {};
//# sourceMappingURL=NetworkSubsystem.d.ts.map