import * as Automerge from "@automerge/automerge";
function mergeArrays(myArrays) {
    // Get the total length of all arrays.
    let length = 0;
    myArrays.forEach((item) => {
        length += item.length;
    });
    // Create a new array with total length and merge all source arrays.
    const mergedArray = new Uint8Array(length);
    let offset = 0;
    myArrays.forEach((item) => {
        mergedArray.set(item, offset);
        offset += item.length;
    });
    return mergedArray;
}
export class StorageSubsystem {
    storageAdapter;
    constructor(storageAdapter) {
        this.storageAdapter = storageAdapter;
    }
    incrementalChanges = {};
    saveIncremental(documentId, doc) {
        const binary = Automerge.getBackend(doc).saveIncremental();
        if (binary && binary.length > 0) {
            if (!this.incrementalChanges[documentId]) {
                this.incrementalChanges[documentId] = 0;
            }
            this.storageAdapter.save(`${documentId}.incremental.${this.incrementalChanges[documentId]}`, binary);
            this.incrementalChanges[documentId]++;
        }
    }
    saveTotal(documentId, doc) {
        const binary = Automerge.save(doc);
        this.storageAdapter.save(`${documentId}.snapshot`, binary);
        for (let i = 0; i < this.incrementalChanges[documentId]; i++) {
            this.storageAdapter.remove(`${documentId}.incremental.${i}`);
        }
        this.incrementalChanges[documentId] = 0;
    }
    async load(storageKey) {
        const result = [];
        let binary = await this.storageAdapter.load(`${storageKey}.snapshot`);
        if (binary && binary.length > 0) {
            result.push(binary);
        }
        let index = 0;
        while ((binary = await this.storageAdapter.load(`${storageKey}.incremental.${index}`))) {
            if (binary && binary.length > 0) {
                result.push(binary);
            }
            index += 1;
        }
        return mergeArrays(result);
    }
    // TODO: make this, you know, good.
    shouldCompact(documentId) {
        return this.incrementalChanges[documentId] >= 20;
    }
    save(documentId, doc) {
        if (this.shouldCompact(documentId)) {
            this.saveTotal(documentId, doc);
        }
        else {
            this.saveIncremental(documentId, doc);
        }
    }
}
