import EventEmitter from "eventemitter3";
import debug from "debug";
const log = debug("CollectionSynchronizer");
import { DocSynchronizer } from "./DocSynchronizer.js";
export class CollectionSynchronizer extends EventEmitter {
    repo;
    peers = {};
    syncPool = {};
    constructor(repo) {
        super();
        this.repo = repo;
    }
    async onSyncMessage(peerId, channelId, message) {
        const documentId = channelId;
        // if we receive a sync message for a document we haven't got in memory,
        // we'll need to register it with the repo and start synchronizing
        const docSynchronizer = await this.fetchDocSynchronizer(documentId);
        log(`onSyncMessage: ${peerId}, ${channelId}, ${message}`);
        docSynchronizer.onSyncMessage(peerId, channelId, message);
        this.__generousPeers().forEach((peerId) => {
            if (!docSynchronizer.peers.includes(peerId)) {
                docSynchronizer.beginSync(peerId);
            }
        });
    }
    async fetchDocSynchronizer(documentId) {
        // TODO: we want a callback to decide to accept offered documents
        if (!this.syncPool[documentId]) {
            const handle = await this.repo.find(documentId);
            this.syncPool[documentId] =
                this.syncPool[documentId] || this.initDocSynchronizer(handle);
        }
        return this.syncPool[documentId];
    }
    initDocSynchronizer(handle) {
        const docSynchronizer = new DocSynchronizer(handle);
        docSynchronizer.on("message", (event) => this.emit("message", event));
        return docSynchronizer;
    }
    async addDocument(documentId) {
        const docSynchronizer = await this.fetchDocSynchronizer(documentId);
        this.__generousPeers().forEach((peerId) => docSynchronizer.beginSync(peerId));
    }
    // need a removeDocument implementation
    // return an array of peers where sharePolicy
    __generousPeers() {
        return Object.entries(this.peers)
            .filter(([, sharePolicy]) => sharePolicy === true)
            .map(([p]) => p);
    }
    addPeer(peerId, sharePolicy) {
        log(`${peerId}, ${sharePolicy}`);
        this.peers[peerId] = sharePolicy;
        if (sharePolicy === true) {
            log(`sharing all open docs`);
            Object.values(this.syncPool).forEach((docSynchronizer) => docSynchronizer.beginSync(peerId));
        }
    }
    removePeer(peerId) {
        log(`removing peer ${peerId}`);
        delete this.peers[peerId];
        for (const docSynchronizer of Object.values(this.syncPool)) {
            docSynchronizer.endSync(peerId);
        }
    }
}
